/*==========================================================================*/
/* DMC Interim out |                find_name.c               |   Utility    */
/*==========================================================================*/
/*
	Name:		find_name.c
	Purpose:	look for a given file in a series of one or more locations
				specified by a circular linked list of path names and return
				the absolute file name if found.
	Usage:		int find_name (req, data, name)
				struct DMC_request req
				struct path_list **data
				char *name
				int ret
				ret = find_name (req, data, name)
	Input:		struct	DMC_request	req;		request file entry
				struct	path_list	**data;		ptr to ptr to circular list of
												   directory paths
				char				*name;		ptr to absolute file name

	Output:		int find_name ()			= 0, file exists
											= 1, file does not exist
											= 2, file status error
				char *name					ptr to absolute file name 
				struct	path_list **data;	new ptr to ptr to circular list of
												   directory paths
	Externals:	Debug - setting of environment variable DEBUG (globals.h)
	Messages:	
	Warnings:	
	Errors:		
	Fatals:		
	Called by:	
	Calls to:	fexist	tests existence of path plus file and returns full name
	Algorithm:	
	Notes:		
	Problems:	
	Debug:		level D_MIN - print out start and finish notices
				level D_MED -
				level D_MAX -
	References:	none
	Language:	ANSI standard C, under Sun OS 3.5
	Revisions:	03/06/89  Kevin MacKenzie  original version
*/

                   /*=====================================*/
/*=================|                                       |=================*/
                   /*=====================================*/
#include "output.h"
   
int find_name (req, data, name)
struct	DMC_request	req;					/* request file entry */
struct	path_list	**data;					/* ptr to ptr to circular list of */
											/*    directory paths */
char				*name;					/* ptr to absolute file name */
{ 
	register struct  path_list   *p;		/* ptr to circular list */
	int		err;							/* result */

	if (Debug >= D_MIN) fprintf (D_OUT, "[find_name] Started.\n");
	p = *data;								/* point to circular list */
	while ( ((err = fexist (p->path, req.entry.file, name)) == 1)
	     && ((p = p->next) != *data) )  ;
	if (err == 0) *data = p;				/* save location in circular list */
	if (Debug >= D_MIN) fprintf (D_OUT, "[find_name] Finished.\n");
	return (err);
}
