/*===========================================================================*/
/* UTIG DMC        |            make_tspan.c               | Utility         */
/*===========================================================================*/
/*
	Name:		make_tspan.c

	Purpose:	allocate and link a timespan node
	Usage:		struct tspan_list *make_tspan( head, tail )
	Input:		struct	tspan_list	**head
				struct	tspan_list	**tail
	Output:		pointer to created node
	Externals:	
	Warnings:	
	Errors:		
	Fatals:		calloc returns no space
	Called by:	
	Calls to:	error_handler()
	Algorithm:	If the list is empty, then both head and tail will be null.
				(eg *head = NULL). If there are any nodes on the list, the
				*head and *tail point to the ends; a node is linked in
				in either case.
	Notes:		
	Problems:	
	References:	
	Language:	
	Revisions:	03/10/89	mark wiederspahn	written
*/

#include "output.h"

struct tspan_list *make_tspan( head, tail )
struct	tspan_list	**head;
struct	tspan_list	**tail;
{
/*
 * if this is the first node, point both head and tail to new node
 * else add one onto the existing list at the end.
 */
	if( *tail == NULL )
	{
		*head = *tail =
			(struct tspan_list *)calloc( 1, sizeof(struct tspan_list) );
		if( *tail == NULL ) error_handler( FATAL,"[make_tspan] no space" );
	}
	else
	{
		(*tail)->next = 
			(struct tspan_list *)calloc( 1, sizeof(struct tspan_list) );
		if( *tail == NULL ) error_handler( FATAL,"[make_tspan] no space" );
		(*tail) = (*tail)->next;
	}

	return( *tail );
}
