/* ================================================================*/
/* DMC Interim out |   podjustdoit.c             |   Main 	   */
/*=================================================================*/
/*
Name:	podjustdoit.c	
Purpose:to find the next continuous series of data records each of
	which lies in whole or in part within the given time span

Usage:	int podjustdoit()

Inputs: none.

Output:	struct input_scan *scan	ptr->data hdr of found time segment
	int	*start_o	ptr->fseek offset to start of data
	int	*end_o		ptr->fseek offset to end of data + 1
	int find_segment ()=0 time segment found
		   =1 last time segment found
		   =2 eof detected before time segment found
		   =3	read error
Messages:	
Warnings:	
Errors:		
Fatals:		
Called by:	
                   /*=====================================*/
/*=================|                                       |=================*/
                   /*=====================================*/
#include <stdio.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/param.h>

#include <dirent.h>
#include "output.h"

#define BOOL int
#define DIM(thing) (sizeof(thing)/sizeof(thing[0]))
#define ELEMENT_DIM(thing)  (sizeof(thing)/DIM(thing))

#define TRIM_STR(s)\
{\
	char *ch_ptr;\
	while((ch_ptr = strchr(s, ' ')) != NULL)\
		*ch_ptr = 0;\
}

/* ------------------------------------------------------------------ */

extern struct path_list *Data_path;
extern struct  DMC_request request;            /* current request */
 


int podjustdoit(ofile, volume_break, tspan_break)
FILE *ofile;
struct  input_time      volume_break;
struct  input_time      tspan_break;


{
	DIR *dirp;
        struct dirent *direntp;

	struct input_time start_volume;
	struct input_time end_volume;
	struct input_time end_tspan;

	char array[10][20];

	struct path_list *next_p, *sav_p;

	struct lvol *first;     

	int err, n, count, nlvol;
	int year, day;


	nlvol = 0;

	/* determine the volume start be searching the data dirs 
	 * and locatinf the earliest file time by file name
	 */
	get_volume_start(&start_volume);

	start_volume.year = cnvt_year(start_volume.year);

	end_volume = add_inputtime(start_volume, volume_break);  
       	end_tspan = add_inputtime(start_volume, tspan_break);    

       	sav_p = next_p = Data_path;

        do
        { 
        	dirp = opendir(next_p->path);

        	if (dirp == NULL)
		{
			next_p = next_p->next;

                	continue;
		}

        	while ((direntp = readdir(dirp)) != NULL)
        	{
			/* filter out the "." and the ".."'s */
                	if ((strcmp(direntp->d_name, ".") == 0) ||
                        	(strcmp(direntp->d_name, "..") == 0))
                        	continue;

			/* build a dummy request entry based on file name
			 * info. Make request for the whole day
			 */

			/* 1st determine if location code is in file name */
			n = split_a(direntp->d_name, &array, ".", DIM(array));

			/* name should be in this format:
			 *		MDJ.CD.VHZ.94.121
			 * which should yeild at least 5 parts.
			 */

			if (n < 5)
				continue;

			strcpy(request.entry.statn, array[0]);
			strcpy(request.entry.network, array[1]);
				strcpy(request.entry.chn, array[2]);

			strcpy(request.entry.location, n == 5 ? "  " : array[3]);

			year = atoi(n == 5 ? array[3] : array[4]);
			
			year = cnvt_year(year);

			day  = atoi(n == 5 ? array[4] : array[5]);

			sprintf(request.entry.start, "%d,%03d", year, day);
			sprintf(request.entry.end, "%d,%03d,23:59:59.9999", year, day);

			strcpy(request.entry.file, direntp->d_name);

			strcpy(request.entry.superfile, Header_path);
		
			strcpy(request.reqstart, request.entry.start);
			strcpy(request.reqend, request.entry.end);
	
			find_data(request, end_tspan, 
					&Lvol_head, &Lvol_tail, 
					&Data_path, &count, &first );

			nlvol += count;

			if( Tspan_tail->count == 0 ) 
					Tspan_tail->start = first;

                	Tspan_tail->count += count;
	
		}

                next_p = next_p->next;

        	closedir(dirp);

        } while (next_p != sav_p);

	err = flush_output_volume(ofile,
					nlvol, 1,
					volume_break,
					tspan_break,
					start_volume,
					end_volume);

}

/* ------------------------------------------------------------------ */
int cnvt_year(y)

{

	if (y < 40)
		/* good until the year 2040,
		 * should be retired by then
		 */
		y += 2000;
	else
		if (y < 100)
			y += 1900;

	return y;

}

/* ------------------------------------------------------------------------ */

int split_a(str, array, thing, maximum)
char *str;
char array[][20];
char *thing;
int maximum;

{
	char wrkstr[100];
	char *ptr;
	int counter;

	counter = 0;

	strncpy(wrkstr, str, sizeof(wrkstr) - 1);

	wrkstr[sizeof(wrkstr)] = 0;

	ptr = strtok(wrkstr, thing);
	
	while (ptr != NULL)
	{
		strcpy(array[counter], ptr);

		counter++;

		if (counter >= maximum)
			return counter;

		ptr = strtok(NULL, thing);
		
	}

	return counter;
	
}

/* ------------------------------------------------------------------ */
int get_volume_start(start_volume)
struct input_time *start_volume;

{
  	struct path_list *next_p;

      	next_p = Data_path;

	start_volume->year 	= 9999;
	start_volume->day 	= 999;
	start_volume->hour 	= 0;
	start_volume->minute 	= 0;
	start_volume->second 	= 0;
	start_volume->fracsec 	= 0;

        do
        { 
		scan_dir(next_p->path, start_volume);

                next_p = next_p->next;
        }
        while (next_p != Data_path);


}

/* ------------------------------------------------------------------ */
int scan_dir(dir, start_volume)
char *dir;
struct input_time *start_volume;

{

        DIR *dirp;
        struct dirent *direntp;
        struct  input_scan scan;

	int daynum, yearnum;
	char *c_ptr;

        dirp = opendir(dir);

	if (dirp == NULL)
		return 0;

        while ((direntp = readdir(dirp)) != NULL)
        {
                /* filter out the "." and the ".."'s */
                if ((strcmp(direntp->d_name, ".") == 0) ||
                        (strcmp(direntp->d_name, "..") == 0))
                        continue;

		/* file name looks like: i.e: NRIL.II.BHZ.94.362
		 * strategy is to peal off the year and day, keeping
		 * the earliest
		 */
	
		c_ptr = strrchr(direntp->d_name, '.');

		if (c_ptr == NULL)
			continue;

		/* move past the '.' */
		daynum = atoi(++c_ptr);

		--c_ptr;

		*c_ptr = 0;


		c_ptr = strrchr(direntp->d_name, '.'); 
 
                if (c_ptr == NULL)
                        continue;

                yearnum = atoi(++c_ptr);

	
		if (start_volume->year > yearnum)
		{
			start_volume->year = yearnum;
			start_volume->day  = daynum;

			continue;
		}
	
		         
		if (start_volume->year == yearnum)
                {
			if (start_volume->day > daynum)
                        	start_volume->day  = daynum;
 
                        continue;
                }

	}

	closedir(dirp);

	return 0;
}


/* ------------------------------------------------------------------ */



int podjustdoit_xxx(count, start_v)
int *count;
struct input_time *start_v;


{
	long now;
	struct tm *tm_now;

	struct input_time end_tspan;

	struct path_list *next_p;

	*count = 0;

	/* get today's date for end of tspan */
	now = time(NULL);               /* seconds since 1970 */
	tm_now = localtime(&now);               /* split it up */
	end_tspan.fracsec = 0;
	end_tspan.second = 0;
	end_tspan.minute = tm_now->tm_min;
	end_tspan.hour = tm_now->tm_hour;
	end_tspan.day = tm_now->tm_yday + 1; /* jan 1 = 0 in unix-speak */
	end_tspan.year = tm_now->tm_year + 1900;


	start_v->year = 9999;

	next_p = Data_path;

	do
	{
		load_into_lvols(next_p->path, &end_tspan, count, start_v);

		next_p = next_p->next;
	}
	while (next_p != Data_path);


}

/* ---------------------------------------------------------------------- */

int load_into_lvols(path, end_tspan, count, start_v)
char *path;
struct input_time *end_tspan;
int *count;
struct input_time *start_v;


{
	DIR *dirp;
	struct dirent *direntp;
	struct  input_scan scan;
	struct input_time endtime;
	int size;
	int err;
	char full_name[MAXPATHLEN];

	dirp = opendir(path);

	while ((direntp = readdir(dirp)) != NULL)
	{
		/* filter out the "." and the ".."'s */

		if ((strcmp(direntp->d_name, ".") == 0) ||
		 	(strcmp(direntp->d_name, "..") == 0))
			continue;

		sprintf(full_name, "%s/%s", path, direntp->d_name);

		err = get_file_info(full_name, 
					&scan, 
					&endtime, 
					&size, 
					end_tspan);

		if (err) 
                {
                        fprintf(stderr, "Error returned from get_file_info()\n");
                        fprintf(stderr, "Unable to continue\n");
 
                        return err;

                }
 

		err = put_lvol(&Lvol_head, &Lvol_tail, 
			 	&scan, full_name, &endtime, 0, size);

		if (Tspan_tail->count == 0)
			Tspan_tail->start = Lvol_tail;
		
		Tspan_tail->count++;

		if (err)
		{
			fprintf(stderr, "Error returned from put_lvol()\n");
			fprintf(stderr, "Unable to continue, malloc error?\n");

			return err;
		}

		(*count)++;

		if (cmp_longtime(scan.time, *start_v) < 0)
			*start_v = scan.time;

	}
	
	closedir(dirp);

	return 0;
}

/* ---------------------------------------------------------------------- */
int get_file_info(fname, scan, endtime, size, end_tspan)
char *fname;
struct input_scan *scan;
struct input_time *endtime;
int *size;
struct input_time *end_tspan;

{
	char buf[4096];			/* max lrecl for now */

	struct input_scan rec_scan;
	struct input_scan sav_rec_scan;
	struct input_time rec_start;      /* ptr->window start time */
	struct input_time rec_end;        /* ptr->window end time */

	int interval, num_bytes;
	struct stat st;
	
	FILE *fptr;

	num_bytes = stat(fname, &st);

	if (num_bytes < 0)
	{
		perror("bad stat");
		return 1;
	}

	*size = st.st_size;

	fptr = fopen(fname, "r");

	if (fptr == NULL)
	{
		fprintf(stderr, "ERROR: get_file_info(): Unable to open: %s for reading\n", fname);

		perror("get_file_info");

		fclose(fptr);

		return 1; 	/* error condition */

	}

	/* obtain lrecl info */
	Lrecl = find_Lrecl(fptr, fname);

	num_bytes = fread(buf, 1, Lrecl, fptr);
	
	if (num_bytes < 0)
	{
		fprintf(stderr, "ERROR: get_file_info(): Unable to read: %s\n", fname);

                perror("get_file_info");

		fclose(fptr);

                return 1;       /* error condition */ 

	}

	get_scaninfo(buf, *end_tspan,
                        &rec_scan, &rec_start, &rec_end, &interval);


	sav_rec_scan = rec_scan;

	/* check out the last record, assumes file times are in order */
	if (fseek(fptr, -Lrecl, 2) == -1)
	{
		fprintf(stderr, "ERROR: get_file_info(): Unable to read: %s\n", fname);
 
                perror("get_file_info");
 
		fclose(fptr);

                return 1;       /* error condition */

	}

        num_bytes = fread(buf, 1, Lrecl, fptr);

        if (num_bytes < 0)
        {
                fprintf(stderr, "ERROR: get_file_info(): Unable to read: %s\n",
fname);
 
                perror("get_file_info");
 
		fclose(fptr);
	
                return 1;       /* error condition */
 
        }
 
        get_scaninfo(buf, *end_tspan,
                        &rec_scan, &rec_start, &rec_end, &interval);
 
 

	/* reset rec_start to the saved value */
	*scan = sav_rec_scan;
	*endtime = rec_end;

	fclose(fptr);
	
	return 0; /* no error */	
}

/* ----------------------------------------------------------------------- */



